/*
 * Include.hpp
 *
 *  Created on: 10 août 2015
 *      Author: aekouali
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include <cstring>
#include <fstream>
#include <sstream>
#include <iostream>
#include <algorithm>
#include <vector>
#include <map>
#include <unordered_map>
#include <cstdlib>
#include <time.h>
#include <algorithm>
#include <iomanip>

using namespace std;

using std::vector;
using std::cout;
using std::cin;
using std::endl;
using std::cerr;
using std::string;
using std::find;

typedef unsigned int uint;

// SCIP Headers

#include <scip/scip.h>
#include <scip/scipdefplugins.h>
#include <scip/cons_linear.h>
#include <scip/scipdefplugins.h>
#include <scip/pub_var.h>
#include <objscip/objscip.h>
#include "objscip/objbranchrule.h"

using namespace scip;

#include "scip_exception.hpp"
#include "wrappers.hpp"
#include "usercons.hpp"

int main(int argc, char* argv[]) {

	SCIP* scip = NULL;
	try {
		SCIP_CALL(SCIPcreate(&scip));
		SCIP_CALL(SCIPincludeDefaultPlugins(scip));
		SCIP_CALL(SCIPsetIntParam(scip, "display/verblevel", 5));
		SCIP_CALL(SCIPcreateProb(scip, "BLP", 0, 0, 0, 0, 0, 0, 0));
		SCIP_CALL(SCIPsetIntParam(scip, "lp/colagelimit", 5));
		SCIP_CALL(SCIPsetObjsense(scip, SCIP_OBJSENSE_MINIMIZE));
		// include the User contraint handler
		SCIP_CALL(SCIPincludeObjConshdlr(scip, new UserConshdlr(scip), TRUE));

		// linear standard constraint
		char con_name[255];
		SCIP_CONS* cons;
		SCIPsnprintf(con_name, 255, "lnr_cons");
		SCIP_CALL(SCIPcreateConsLinear(scip, &cons, con_name, 0, NULL, NULL, 3, /* lhs */
		3, /* rhs */
		true, /* initial */
		true, /* separate */
		true, /* enforce */
		true, /* check */
		true, /* propagate */
		false, /* local */
		true, /* modifiable */
		false, /* dynamic */
		false, /* removable */
		false));
		/* add constraint to scip */
		SCIP_CALL(SCIPaddCons(scip, cons));

		// User constraint
		SCIP_CONS* consuser;
		uint first = 1;
		uint second = 2;
		SCIPsnprintf(con_name, 255, "UserCons-%i-%i", first, second);
		SCIP_CALL(SCIPcreateUserCons(scip, &consuser, con_name, first, second));
		SCIP_CALL(SCIPaddCons(scip, consuser));

		// adding the initial variables
		vector<SCIP_VAR*> mvars;
		for (uint i = 0; i < 3; i++) {
			// create initial variable
			char var_name[255];
			SCIP_VAR* var;
			SCIPsnprintf(var_name, 255, "X_%d", SCIPgetNVars(scip));
			SCIP_CALL(SCIPcreateVar(scip, &var, var_name, 0.0, // lower bound
					1.0, // upper bound
					-1, // objective
					SCIP_VARTYPE_BINARY, // variable type
					false, false, 0, 0, 0, 0, 0));
			SCIPchgVarUbLazy(scip, var, 1);
			SCIP_CALL(SCIPaddVar(scip, var));
			SCIP_CALL(SCIPaddCoefLinear(scip, cons, var, 1.0));
			mvars.push_back(var);
			SCIP_CALL(SCIPreleaseVar(scip, &var));
		}
		SCIPwriteOrigProblem(scip, "BLP.lp", "lp", true);

		// solve
		SCIPsetMessagehdlrQuiet(scip, false);
		SCIP_CALL(SCIPsolve(scip));

		SCIP_SOL* sol = SCIPgetBestSol(scip);
		uint nvars = SCIPgetNVars(scip);
		SCIP_VAR** vars = SCIPgetVars(scip);
		cout << "Number of all variables: " << nvars << endl;
		cout << "Obj: " << SCIPgetSolOrigObj(scip, sol) << endl;
		for (uint i = 0; i < nvars; i++) {
			SCIP_Real val = SCIPgetSolVal(scip, sol, vars[i]);
			if (SCIPvarIsBinary(vars[i]) and not SCIPisZero(scip, val)) {
				cout << SCIPvarGetName(vars[i]) << ": " << val << endl;
			}
		}

		// Deinitialization
		SCIP_CALL(SCIPfree(&scip));
		BMScheckEmptyMemory();

	} catch (SCIPException& exc) {
		cerr << exc.what() << endl;
		exit(exc.getRetcode());
	}
	return EXIT_SUCCESS;
}
