/*
 * usercons.cpp
 *
 *  Created on: 10 août 2015
 *      Author: aekouali
 */

/*
 * cons_mlcl.cpp
 *
 *  Created on: 3 avr. 2015
 *      Author: aekouali
 */

#include "usercons.hpp"

struct SCIP_ConsData {
	unsigned first;
	unsigned second;
};

/** frees specific constraint data */
SCIP_DECL_CONSDELETE(UserConshdlr::scip_delete) {
	assert(consdata != NULL);
	return SCIP_OKAY;
}

/** transforms constraint data into data belonging to the transformed problem */
SCIP_DECL_CONSTRANS(UserConshdlr::scip_trans) {
	SCIP_CONSDATA* sourcedata = NULL;
	SCIP_CONSDATA* targetdata = NULL;

	sourcedata = SCIPconsGetData(sourcecons);
	assert(sourcedata != NULL);

	SCIP_CALL(SCIPallocMemory(scip, &targetdata));
	targetdata->first = sourcedata->first;
	targetdata->second = sourcedata->second;

	/* create target constraint */
	SCIP_CALL(
			SCIPcreateCons(scip, targetcons, SCIPconsGetName(sourcecons),
					conshdlr, targetdata, SCIPconsIsInitial(sourcecons),
					SCIPconsIsSeparated(sourcecons),
					SCIPconsIsEnforced(sourcecons),
					SCIPconsIsChecked(sourcecons),
					SCIPconsIsPropagated(sourcecons),
					SCIPconsIsLocal(sourcecons),
					SCIPconsIsModifiable(sourcecons),
					SCIPconsIsDynamic(sourcecons),
					SCIPconsIsRemovable(sourcecons),
					SCIPconsIsStickingAtNode(sourcecons)));

	return SCIP_OKAY;
}

SCIP_DECL_CONSENFOLP(UserConshdlr::scip_enfolp) {
	*result = SCIP_INFEASIBLE;
	return SCIP_OKAY;
}

SCIP_DECL_CONSENFOPS(UserConshdlr::scip_enfops) {
	*result = SCIP_INFEASIBLE;
	return SCIP_OKAY;
}

SCIP_DECL_CONSCHECK(UserConshdlr::scip_check) {
	*result = SCIP_INFEASIBLE;
	return SCIP_OKAY;
}

SCIP_DECL_CONSLOCK(UserConshdlr::scip_lock) {
	return SCIP_OKAY;
}

SCIP_DECL_CONSPROP(UserConshdlr::scip_prop) {
	*result = SCIP_DIDNOTFIND;
	return SCIP_OKAY;
}

SCIP_RETCODE SCIPcreateUserCons(SCIP* scip, SCIP_CONS** cons, const char* name,
		uint first, /**< the first information */
		uint second /**< the second information */
		) {
	SCIP_CONSDATA* consdata = NULL;

	/* create constraint data */
	SCIP_CALL(SCIPallocMemory( scip, &consdata));
	consdata->first = first;
	consdata->second = second;

	SCIP_CONSHDLR* conshdlr = NULL;

	/* find the User constraint handler */
	conshdlr = SCIPfindConshdlr(scip, "User");
	if (conshdlr == NULL) {
		SCIPerrorMessage("User constraint handler not found\n")
;
		return SCIP_PLUGINNOTFOUND;
	}
	/* create constraint */
	SCIP_CALL(
			SCIPcreateCons(scip, cons, name, conshdlr, consdata, FALSE, FALSE, FALSE, FALSE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE ));

	return SCIP_OKAY;
}
